package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.Likes;
import at.ac.uibk.gitsearch.repository.jpa.LikesRepository;
import java.util.List;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link Likes}.
 */
@Service
@Transactional
public class LikesService {

    private final Logger log = LoggerFactory.getLogger(LikesService.class);

    private final LikesRepository likesRepository;

    public LikesService(LikesRepository likesRepository) {
        this.likesRepository = likesRepository;
    }

    /**
     * Save a likes.
     *
     * @param likes the entity to save.
     * @return the persisted entity.
     */
    public Likes save(Likes likes) {
        log.debug("Request to save Likes : {}", likes);
        return likesRepository.save(likes);
    }

    /**
     * Get all the likes.
     *
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<Likes> findAll() {
        return likesRepository.findAll();
    }

    public long countAll() {
        return likesRepository.count();
    }

    /**
     * Get one likes by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<Likes> findOne(Long id) {
        log.debug("Request to get Likes : {}", id);
        return likesRepository.findById(id);
    }

    /**
     * Delete the likes by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete Likes : {}", id);
        likesRepository.deleteById(id);
    }

    public void deleteByUserIDandExerciseID(Integer userID, String exerciseID) {
        Likes like = findLikesByUserIDandExerciseID(userID, exerciseID);
        if (like.getId() != null) {
            delete(like.getId());
        }
    }

    @Transactional(readOnly = true)
    public Likes findLikesByUserIDandExerciseID(Integer userID, String exerciseID) {
        log.debug("Request to get Likes by User id and project id : {} {}", userID, exerciseID);
        return likesRepository.findLikesByUserIDandExerciseID(userID, exerciseID);
    }

    @Transactional(readOnly = true)
    public Integer findNumberOfLikesByExerciseID(String exerciseID) {
        log.debug("Request to get number of Likes by project id : {}", exerciseID);
        return likesRepository.findNumberOfLikesByExerciseID(exerciseID);
    }
}
