package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.SavedSearches;
import at.ac.uibk.gitsearch.repository.jpa.SavedSearchesRepository;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;
import at.ac.uibk.gitsearch.service.mapper.SavedSearchesMapper;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link SavedSearches}.
 */
@Service
@Transactional
public class SavedSearchesService {

    private final Logger log = LoggerFactory.getLogger(SavedSearchesService.class);

    private final SavedSearchesRepository savedSearchesRepository;

    private final SavedSearchesMapper savedSearchesMapper;

    public SavedSearchesService(SavedSearchesRepository savedSearchesRepository, SavedSearchesMapper savedSearchesMapper) {
        this.savedSearchesRepository = savedSearchesRepository;
        this.savedSearchesMapper = savedSearchesMapper;
    }

    /**
     * Save a savedSearches.
     *
     * @param savedSearchesDTO the entity to save.
     * @return the persisted entity.
     */
    public SavedSearchesDTO save(SavedSearchesDTO savedSearchesDTO) {
        log.debug("Request to save SavedSearches : {}", savedSearchesDTO);
        SavedSearches savedSearches = savedSearchesMapper.toEntity(savedSearchesDTO);
        savedSearches = savedSearchesRepository.save(savedSearches);
        return savedSearchesMapper.toDto(savedSearches);
    }

    /**
     * Update a savedSearches.
     *
     * @param savedSearchesDTO the entity to save.
     * @return the persisted entity.
     */
    public SavedSearchesDTO update(SavedSearchesDTO savedSearchesDTO) {
        log.debug("Request to update SavedSearches : {}", savedSearchesDTO);
        SavedSearches savedSearches = savedSearchesMapper.toEntity(savedSearchesDTO);
        savedSearches = savedSearchesRepository.save(savedSearches);
        return savedSearchesMapper.toDto(savedSearches);
    }

    /**
     * Partially update a savedSearches.
     *
     * @param savedSearchesDTO the entity to update partially.
     * @return the persisted entity.
     */
    public Optional<SavedSearchesDTO> partialUpdate(SavedSearchesDTO savedSearchesDTO) {
        log.debug("Request to partially update SavedSearches : {}", savedSearchesDTO);

        return savedSearchesRepository
            .findById(savedSearchesDTO.getId())
            .map(existingSavedSearches -> {
                savedSearchesMapper.partialUpdate(existingSavedSearches, savedSearchesDTO);

                return existingSavedSearches;
            })
            .map(savedSearchesRepository::save)
            .map(savedSearchesMapper::toDto);
    }

    /**
     * Get all the savedSearches.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<SavedSearchesDTO> findAll(Pageable pageable) {
        log.debug("Request to get all SavedSearches");
        return savedSearchesRepository.findAll(pageable).map(savedSearchesMapper::toDto);
    }

    /**
     * Get one savedSearches by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<SavedSearchesDTO> findOne(Long id) {
        log.debug("Request to get SavedSearches : {}", id);
        return savedSearchesRepository.findById(id).map(savedSearchesMapper::toDto);
    }

    /**
     * Delete the savedSearches by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete SavedSearches : {}", id);
        savedSearchesRepository.deleteById(id);
    }
}
