package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.repository.jpa.ReviewHistoryEntryRepository;
import at.ac.uibk.gitsearch.service.ReviewHistoryEntryQueryService;
import at.ac.uibk.gitsearch.service.ReviewHistoryEntryService;
import at.ac.uibk.gitsearch.service.criteria.ReviewHistoryEntryCriteria;
import at.ac.uibk.gitsearch.service.dto.ReviewHistoryEntryDTO;
import at.ac.uibk.gitsearch.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link at.ac.uibk.gitsearch.domain.ReviewHistoryEntry}.
 */
@RestController
@RequestMapping("/api")
@SuppressWarnings({ "PMD.AvoidDuplicateLiterals" })
public class ReviewHistoryEntryResource {

    private final Logger log = LoggerFactory.getLogger(ReviewHistoryEntryResource.class);

    private static final String ENTITY_NAME = "reviewHistoryEntry";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final ReviewHistoryEntryService reviewHistoryEntryService;

    private final ReviewHistoryEntryRepository reviewHistoryEntryRepository;

    private final ReviewHistoryEntryQueryService reviewHistoryEntryQueryService;

    public ReviewHistoryEntryResource(
        ReviewHistoryEntryService reviewHistoryEntryService,
        ReviewHistoryEntryRepository reviewHistoryEntryRepository,
        ReviewHistoryEntryQueryService reviewHistoryEntryQueryService
    ) {
        this.reviewHistoryEntryService = reviewHistoryEntryService;
        this.reviewHistoryEntryRepository = reviewHistoryEntryRepository;
        this.reviewHistoryEntryQueryService = reviewHistoryEntryQueryService;
    }

    /**
     * {@code POST  /review-history-entries} : Create a new reviewHistoryEntry.
     *
     * @param reviewHistoryEntryDTO the reviewHistoryEntryDTO to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new reviewHistoryEntryDTO, or with status {@code 400 (Bad Request)} if the reviewHistoryEntry has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/review-history-entries")
    public ResponseEntity<ReviewHistoryEntryDTO> createReviewHistoryEntry(@Valid @RequestBody ReviewHistoryEntryDTO reviewHistoryEntryDTO)
        throws URISyntaxException {
        log.debug("REST request to save ReviewHistoryEntry : {}", reviewHistoryEntryDTO);
        if (reviewHistoryEntryDTO.getId() != null) {
            throw new BadRequestAlertException("A new reviewHistoryEntry cannot already have an ID", ENTITY_NAME, "idexists");
        }
        ReviewHistoryEntryDTO result = reviewHistoryEntryService.save(reviewHistoryEntryDTO);
        return ResponseEntity
            .created(new URI("/api/review-history-entries/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /review-history-entries/:id} : Updates an existing reviewHistoryEntry.
     *
     * @param id the id of the reviewHistoryEntryDTO to save.
     * @param reviewHistoryEntryDTO the reviewHistoryEntryDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated reviewHistoryEntryDTO,
     * or with status {@code 400 (Bad Request)} if the reviewHistoryEntryDTO is not valid,
     * or with status {@code 500 (Internal Server Error)} if the reviewHistoryEntryDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/review-history-entries/{id}")
    public ResponseEntity<ReviewHistoryEntryDTO> updateReviewHistoryEntry(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody ReviewHistoryEntryDTO reviewHistoryEntryDTO
    ) throws URISyntaxException {
        log.debug("REST request to update ReviewHistoryEntry : {}, {}", id, reviewHistoryEntryDTO);
        if (reviewHistoryEntryDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, reviewHistoryEntryDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!reviewHistoryEntryRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        ReviewHistoryEntryDTO result = reviewHistoryEntryService.save(reviewHistoryEntryDTO);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, reviewHistoryEntryDTO.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /review-history-entries/:id} : Partial updates given fields of an existing reviewHistoryEntry, field will ignore if it is null
     *
     * @param id the id of the reviewHistoryEntryDTO to save.
     * @param reviewHistoryEntryDTO the reviewHistoryEntryDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated reviewHistoryEntryDTO,
     * or with status {@code 400 (Bad Request)} if the reviewHistoryEntryDTO is not valid,
     * or with status {@code 404 (Not Found)} if the reviewHistoryEntryDTO is not found,
     * or with status {@code 500 (Internal Server Error)} if the reviewHistoryEntryDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/review-history-entries/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<ReviewHistoryEntryDTO> partialUpdateReviewHistoryEntry(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody ReviewHistoryEntryDTO reviewHistoryEntryDTO
    ) throws URISyntaxException {
        log.debug("REST request to partial update ReviewHistoryEntry partially : {}, {}", id, reviewHistoryEntryDTO);
        if (reviewHistoryEntryDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, reviewHistoryEntryDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!reviewHistoryEntryRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<ReviewHistoryEntryDTO> result = reviewHistoryEntryService.partialUpdate(reviewHistoryEntryDTO);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, reviewHistoryEntryDTO.getId().toString())
        );
    }

    /**
     * {@code GET  /review-history-entries} : get all the reviewHistoryEntries.
     *
     * @param criteria the criteria which the requested entities should match.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of reviewHistoryEntries in body.
     */
    @GetMapping("/review-history-entries")
    public ResponseEntity<List<ReviewHistoryEntryDTO>> getAllReviewHistoryEntries(ReviewHistoryEntryCriteria criteria) {
        log.debug("REST request to get ReviewHistoryEntries by criteria: {}", criteria);
        List<ReviewHistoryEntryDTO> entityList = reviewHistoryEntryQueryService.findByCriteria(criteria);
        return ResponseEntity.ok().body(entityList);
    }

    /**
     * {@code GET  /review-history-entries/count} : count all the reviewHistoryEntries.
     *
     * @param criteria the criteria which the requested entities should match.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the count in body.
     */
    @GetMapping("/review-history-entries/count")
    public ResponseEntity<Long> countReviewHistoryEntries(ReviewHistoryEntryCriteria criteria) {
        log.debug("REST request to count ReviewHistoryEntries by criteria: {}", criteria);
        return ResponseEntity.ok().body(reviewHistoryEntryQueryService.countByCriteria(criteria));
    }

    /**
     * {@code GET  /review-history-entries/:id} : get the "id" reviewHistoryEntry.
     *
     * @param id the id of the reviewHistoryEntryDTO to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the reviewHistoryEntryDTO, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/review-history-entries/{id}")
    public ResponseEntity<ReviewHistoryEntryDTO> getReviewHistoryEntry(@PathVariable Long id) {
        log.debug("REST request to get ReviewHistoryEntry : {}", id);
        Optional<ReviewHistoryEntryDTO> reviewHistoryEntryDTO = reviewHistoryEntryService.findOne(id);
        return ResponseUtil.wrapOrNotFound(reviewHistoryEntryDTO);
    }

    /**
     * {@code DELETE  /review-history-entries/:id} : delete the "id" reviewHistoryEntry.
     *
     * @param id the id of the reviewHistoryEntryDTO to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/review-history-entries/{id}")
    public ResponseEntity<Void> deleteReviewHistoryEntry(@PathVariable Long id) {
        log.debug("REST request to delete ReviewHistoryEntry : {}", id);
        reviewHistoryEntryService.delete(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
