package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.ReviewHistoryEntryService;
import at.ac.uibk.gitsearch.service.ReviewQueryService;
import at.ac.uibk.gitsearch.service.ReviewService;
import at.ac.uibk.gitsearch.service.UserService;
import at.ac.uibk.gitsearch.service.criteria.ReviewCriteria;
import at.ac.uibk.gitsearch.service.dto.ReviewDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewHistoryEntryDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO.ReviewStatisticsDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRequest;
import at.ac.uibk.gitsearch.service.dto.UserDTO;
import java.util.List;
import java.util.Optional;
import javax.transaction.Transactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api")
@Transactional
@SuppressWarnings({ "PMD.TooManyMethods" })
public class ReviewResource {

    private final Logger log = LoggerFactory.getLogger(ReviewResource.class);

    private final ReviewService reviewService;

    private final UserService userService;

    private final ReviewQueryService reviewQueryService;

    private final ReviewHistoryEntryService reviewHistoryEntryService;

    private static final String ADMIN_ROLE = "hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")";

    private static final String USER_ROLE = "hasAuthority(\"" + AuthoritiesConstants.USER + "\")";

    public ReviewResource(
        ReviewService reviewService,
        UserService userService,
        ReviewQueryService reviewQueryService,
        ReviewHistoryEntryService reviewHistoryEntryService
    ) {
        this.reviewService = reviewService;
        this.userService = userService;
        this.reviewQueryService = reviewQueryService;
        this.reviewHistoryEntryService = reviewHistoryEntryService;
    }

    @PostMapping("/review/create")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<Void> createReview(@RequestBody ReviewRequest reviewRequest) {
        log.debug("REST request to create a review");
        reviewService.createReview(reviewRequest);
        return ResponseEntity.ok().build();
    }

    @PutMapping("/review")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<Void> updateReview(@RequestBody ReviewDTO review) {
        log.debug("REST request to update a review");
        reviewService.updateReview(review);
        return ResponseEntity.ok().build();
    }

    @GetMapping("/review/{id}")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<ReviewDTO> getReview(@PathVariable Long id) {
        log.debug("REST request to get review with id: {}", id);
        return ResponseEntity.ok(reviewService.getReview(id));
    }

    @GetMapping("/review/{id}/resource")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<String> getReviewResource(@PathVariable Long id) {
        return ResponseEntity.ok(reviewService.getReview(id).getResource());
    }

    //GET ALL REVIEWS FROM A USER
    @GetMapping("/review/allByUser")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<List<ReviewDTO>> getReviewsByUser() {
        return ResponseEntity.ok(reviewService.getReviewsByUser());
    }

    /**
     * get statistics for user
     *
     * @return
     */
    @GetMapping("/review/countByUser")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<List<ReviewDTO>> countReviewsByUser() {
        return ResponseEntity.ok(reviewService.getReviewsByUser());
    }

    //UPDATE A REVIEW BY A USER
    @PutMapping("/review/updateByUser")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<Void> updateReviewRatingByUser(@RequestBody ReviewDTO review) throws IllegalAccessException {
        log.debug("REST request to update a review");
        reviewService.updateReviewRatingByUser(review);
        return ResponseEntity.ok().build();
    }

    /**
     * {@code GET  /reviews} : get all the reviews.
     *
     * @param criteria the criteria which the requested entities should match.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of reviews in body.
     */
    @GetMapping("/review")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<List<ReviewDTO>> getAllReviews(ReviewCriteria criteria) {
        log.debug("REST request to get Reviews by criteria: {}", criteria);
        List<ReviewDTO> entityList = reviewQueryService.findByCriteria(criteria);
        return ResponseEntity.ok().body(entityList);
    }

    @GetMapping("/review/all")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<List<ReviewDTO>> getAllReviews() {
        log.debug("REST request to get all reviews");
        return ResponseEntity.ok(reviewService.getAllReviews());
    }

    @DeleteMapping("/review/{id}")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<Void> deleteReview(@PathVariable Long id) {
        log.debug("REST request to delete review: {}", id);
        reviewService.deleteReview(id);
        return ResponseEntity.ok().build();
    }

    @PostMapping("/requestReview")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<Void> requestReviewForExercise(@RequestBody ReviewRequest reviewRequest) {
        log.debug("REST request to request a review for exercise: {}", reviewRequest.getResource());
        boolean created = reviewService.requestReviewForExercise(reviewRequest);
        return created ? ResponseEntity.ok().build() : ResponseEntity.badRequest().build();
    }

    @PostMapping("/review/notifyReviewers")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<Void> notifyReviewers(@RequestBody ReviewDTO review) {
        log.debug("REST request to notify reviewers of resource: {}", review.getResource());
        reviewService.updateStatus(review.getId(), ReviewStatus.REVIEW_IMPROVED);
        reviewService.notifyReviewers(
            review.getId(),
            "The review for " + review.getResource() + " has been updated. Please verify and update the status if applicable."
        );
        return ResponseEntity.ok().build();
    }

    @PostMapping("/review/rewardBadge")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<Void> rewardBadge(@RequestBody ReviewDTO review) {
        log.debug("REST request to award badge for resource: {}", review.getResource());
        reviewService.rewardBadge(review.getId());
        return ResponseEntity.ok().build();
    }

    @PostMapping("/review/removeBadge")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<Void> removeBadge(@RequestBody ReviewDTO review) {
        log.debug("REST request to remove badge for resource: {}", review.getResource());
        reviewService.removeBadge(review.getId());
        return ResponseEntity.ok().build();
    }

    @PostMapping("/review/changeStatus")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<UserDTO> changeStatus(@RequestBody boolean status) {
        log.debug("REST request to change status of reviewer: {}", status);
        Optional<User> user = userService.getUserWithAuthorities();
        if (user.isPresent()) {
            user.get().setReviewingEnabled(status);
            userService.updateUser(user.get());
        }
        return ResponseEntity.ok().build();
    }

    //Get all reviews with statistics
    @GetMapping("/review/allWithStatistics")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<List<ReviewDTO>> getAllReviewsWithStatistics() {
        log.debug("REST request to get all reviews with statistics");
        return ResponseEntity.ok(reviewService.getAllReviewsWithStatistics());
    }

    @PostMapping("/review/countAllByUser")
    @PreAuthorize(USER_ROLE)
    public ResponseEntity<ReviewStatisticsDTO> countAllByUser() {
        Optional<User> user = userService.getUserWithAuthorities();
        if (user.isPresent()) {
            final ReviewStatisticsDTO statusCounts = reviewService.getReviewStatusCountsByUser(user.get());
            return ResponseEntity.ok(statusCounts);
        }
        return ResponseEntity.notFound().build();
    }

    //Get all reviews with statistics
    @GetMapping("/review/reviewRatings")
    @PreAuthorize(ADMIN_ROLE)
    public ResponseEntity<List<ReviewRatingDTO>> getAllReviewRatings() {
        log.debug("REST request to get all reviews with statistics");
        return ResponseEntity.ok(reviewService.getAllReviewRatings());
    }

    /**
     * {@code GET  /review-history-entries} : get all the reviewHistoryEntries for
     * current review and current user.
     * The history entries are anonymized, i.e. no user id is transfered
     *
     * @param reviewId       the id of the review.
     * @param reviewRatingId the id of the reviewRating ( may be null )
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list
     *         of reviewHistoryEntries in body.
     */
    @GetMapping("/review/reviewHistory")
    public ResponseEntity<List<ReviewHistoryEntryDTO>> getAllReviewHistoryEntries(Long reviewId, Long reviewRatingId) {
        log.debug("REST request to get ReviewHistoryEntries by review id {} and rating id {}", reviewId, reviewRatingId);
        final List<ReviewHistoryEntryDTO> historyEntries = reviewHistoryEntryService.findByReviewReviewRating(reviewId, reviewRatingId);
        // anonymize
        historyEntries.forEach(entry -> entry.setUser(0L));
        return ResponseEntity.ok().body(historyEntries);
    }
}
